#if defined(_MSC_VER)
#define NOMINMAX
#pragma once
#endif

#ifndef PBRT_SAMPLERS_GAUSSJITTER_H
#define PBRT_SAMPLERS_GAUSSJITTER_H

// samplers/gaussjitter.h*
#include "sampler.h"
#include "rng.h"

namespace pbrt {

// GaussJitterSampler Declarations
class GaussJitterSampler : public PixelSampler {
  public:
    // GaussJitterSampler Public Methods
    GaussJitterSampler(int xPixelSamples, int yPixelSamples, bool jitterSamples,
                      int nSampledDimensions)
        : PixelSampler(xPixelSamples * yPixelSamples, nSampledDimensions),
          xPixelSamples(xPixelSamples),
          yPixelSamples(yPixelSamples),
          jitterSamples(jitterSamples) {}
    void StartPixel(const Point2i &);
    std::unique_ptr<Sampler> Clone(int seed);

  private:
    // GaussSampler Private Data
    const int xPixelSamples, yPixelSamples;
    const bool jitterSamples;
};

GaussJitterSampler *CreateGaussJitterSampler(const ParamSet &params);

}  // namespace pbrt

#endif  // PBRT_SAMPLERS_GAUSSJITTER_H
